@@ -1,3 +1,24 @@
+addSignificance <- function (
+    
+  model_fits,
+  sign_models
+  
+) {
+  
+  names(sign_models) <- NULL
+  
+  model_fits_out <- lapply(seq_along(model_fits), function (i) {
+    
+    c(model_fits[[i]], significant = sign_models[i])
+    
+  })
+  
+  attributes(model_fits_out) <- attributes(model_fits)
+  
+  return (model_fits_out)
+  
+}
+
 #' @title assessDesign
 #'
 #' @description This function performs simulation based trial design evaluations for a set of specified dose-response models
@@ -10,17 +31,18 @@
 #' @param alpha_crit_val (Un-adjusted) Critical value to be used for the MCP testing step. Passed to the getCritProb() function for the calculation of adjusted critical values (on the probability scale). Default is 0.05.
 #' @param modeling Boolean variable defining whether the Mod part of Bayesian MCP-Mod will be performed in the assessment. More heavy on resources. Default FALSE.
 #' @param simple Boolean variable defining whether simplified fit will be applied. Passed to the getModelFits function. Default FALSE.
+#' @param avg_fit Boolean variable, defining whether an average fit (based on generalized AIC weights) should be performed in addition to the individual models. Default TRUE.
 #' @param reestimate Boolean variable defining whether critical value should be calculated with re-estimated contrasts (see getCritProb function for more details). Default FALSE
 #' @param contr An object of class 'optContr' as created by the getContr() function. Allows specification of a fixed contrasts matrix. Default NULL
 #' @param dr_means A vector, allows specification of  individual (not model based) assumed effects per dose group. Default NULL
-#'
+#' @param delta A numeric value for the threshold Delta for the MED assessment. If NULL, no MED assessment is performed. Default NULL.
+#' @param evidence_level A numeric value between 0 and 1 for the evidence level gamma for the MED assessment. Only required for Bayesian MED assessment, see ?getMED for details. Default NULL.
+#' @param med_selection A string, either "avgFit" or "bestFit", for the method of MED selection. Default "avgFit".
+#' 
 #' @return Returns success probabilities for the different assumed dose-response shapes, attributes also includes information around average success rate (across all assumed models) and prior Effective sample size
 #'
 #' @examples
-#' if (interactive()) { # takes typically > 5 seconds
-#'
 #' mods <- DoseFinding::Mods(linear      = NULL,
-#'                           linlog      = NULL,
 #'                           emax        = c(0.5, 1.2),
 #'                           exponential = 2,
 #'                           doses       = c(0, 0.5, 2,4, 8),
@@ -38,32 +60,67 @@
 #'   mods        = mods,
 #'   prior_list  = prior_list,
 #'   sd          = sd,
-#'   n_sim       = 1e2) # speed up exammple run time
+#'   n_sim       = 1e2) # speed up example run time
 #'
 #' success_probabilities
+#' 
+#' if (interactive()) { # takes typically > 5 seconds
+#' 
+#' # with MED estimation without bootstrapping
+#' # see ?getMED for details
+#' 
+#' success_probabilities <- assessDesign(
+#'   n_patients     = n_patients,
+#'   mods           = mods,
+#'   prior_list     = prior_list,
+#'   sd             = sd,
+#'   modeling       = TRUE,
+#'   n_sim          = 10, # speed up example run time
+#'   delta          = 7)
+#'   
+#'   success_probabilities
+#'   
+#' # with MED estimation with bootstrapping
+#' 
+#' success_probabilities <- assessDesign(
+#'   n_patients     = n_patients,
+#'   mods           = mods,
+#'   prior_list     = prior_list,
+#'   sd             = sd,
+#'   modeling       = TRUE,
+#'   n_sim          = 10, # speed up example run time
+#'   delta          = 7,
+#'   evidence_level = 0.8)
+#'   
+#'   success_probabilities
 #'
 #' }
 #'
 #' @export
 assessDesign <- function (
-
+    
   n_patients,
   mods,
   prior_list,
-
+  
   sd,
-
+  
   n_sim          = 1e3,
   alpha_crit_val = 0.05,
   modeling       = FALSE,
   simple         = TRUE,
+  avg_fit        = TRUE,
   reestimate     = FALSE,
-
+  
   contr          = NULL,
-  dr_means       = NULL
-
+  dr_means       = NULL,
+  
+  delta          = NULL,
+  evidence_level = NULL,
+  med_selection  = c("avgFit", "bestFit")
+  
 ) {
-
+  
   checkmate::assert_vector(n_patients, len = length(attr(mods, "doses")), any.missing = FALSE)
   checkmate::assert_double(n_patients, lower = 2, upper = Inf)
   checkmate::check_class(mods, classes = "Mods")
@@ -71,12 +128,14 @@ assessDesign <- function (
   # sensitive to how DoseFinding labels their attributes for "Mods" class
   checkmate::check_double(n_sim, lower = 1, upper = Inf)
   checkmate::check_double(alpha_crit_val, lower = 0, upper = 1)
-  checkmate::check_logical(simple)
   checkmate::check_logical(modeling)
+  
   # TODO: check that prior_list has 'sd_tot' attribute, and that it's numeric # this is not applicable at the moment
-
+  
+  modeling <- ifelse(!is.null(delta), TRUE, modeling)
+  
   dose_levels <- attr(mods, "doses")
-
+  
   data <- simulateData(
     n_patients  = n_patients,
     dose_levels = dose_levels,
@@ -84,61 +143,66 @@ assessDesign <- function (
     mods        = mods,
     n_sim       = n_sim,
     dr_means    = dr_means)
-
+  
   model_names <- colnames(data)[-c(1:3)]
-
+  
   crit_prob_adj <- getCritProb(
     mods           = mods,
     dose_levels    = dose_levels,
     dose_weights   = n_patients,
     alpha_crit_val = alpha_crit_val)
-
+  
   if (!reestimate & is.null(contr)) {
-
+    
     contr <- getContr(
       mods           = mods,
       dose_levels    = dose_levels,
       dose_weights   = n_patients,
       prior_list     = prior_list)
-
+    
   }
-
+  
   eval_design <- lapply(model_names, function (model_name) {
-
+    
     posterior_list <- getPosterior(
       data       = getModelData(data, model_name),
       prior_list = prior_list)
-
+    
     if (reestimate & is.null(contr)) {
-
+      
       post_sds <- sapply(posterior_list, function (post) summary(post)[, 2])
-
+      
       contr <- apply(post_sds, 2, function (post_sd) getContr(
         mods           = mods,
         dose_levels    = dose_levels,
         sd_posterior   = post_sd))
-
+      
     }
     
-    if (identical(modeling, FALSE)) {
+    if (modeling) {
+      
+      b_mcp_mod <- performBayesianMCPMod(
+        posterior_list = posterior_list,
+        contr          = contr,
+        crit_prob_adj  = crit_prob_adj,
+        simple         = simple,
+        avg_fit        = avg_fit,
+        delta          = delta,
+        evidence_level = evidence_level,
+        med_selection  = med_selection,
+        n_samples      = n_sim)
+      
+    } else {
       
       b_mcp <- performBayesianMCP(
         posterior_list = posterior_list,
         contr          = contr,
         crit_prob_adj  = crit_prob_adj)
       
-    } else {
-      
-      b_mcp_mod <- performBayesianMCPMod(
-        posterior_list = posterior_list,
-        contr          = contr,
-        crit_prob_adj  = crit_prob_adj,
-        simple         = simple)
-      
     }
-
+    
   })
-
+  
   avg_success_rate <- mean(sapply(eval_design, function (x) {
     
     ifelse(identical(modeling, FALSE),
@@ -148,7 +212,7 @@ assessDesign <- function (
   }))
   
   names(eval_design) <- model_names
-
+  
   attr(eval_design, "avgSuccessRate") <- avg_success_rate
   attr(eval_design, "placEff")        <- ifelse(test = is.null(dr_means),
                                                 yes  = attr(mods, "placEff"),
@@ -158,15 +222,58 @@ assessDesign <- function (
                                                 no   = diff(range(dr_means)))
   attr(eval_design, "sampleSize")     <- n_patients
   attr(eval_design, "priorESS")       <- round(getESS(prior_list), 1)
-
+  
   return (eval_design)
+  
+}
 
+BayesMCPi <- function (
+    
+  posterior_i,
+  contr,
+  crit_prob_adj
+  
+) {
+  
+  getPostProb <- function (
+    
+    contr_j,     # j: dose level
+    post_combs_i # i: simulation outcome
+    
+  ) {
+    
+    ## Test statistic = sum over all components of
+    ## posterior weight * normal probability distribution of
+    ## critical values for doses * estimated mean / sqrt(product of critical values for doses)
+    
+    ## Calculation for each component of the posterior
+    contr_theta   <- apply(post_combs_i$means, 1, `%*%`, contr_j)
+    contr_var     <- apply(post_combs_i$vars, 1, `%*%`, contr_j^2)
+    contr_weights <- post_combs_i$weights
+    
+    ## P(c_m * theta > 0 | Y = y) for a shape m (and dose j)
+    post_probs <- sum(contr_weights * stats::pnorm(contr_theta / sqrt(contr_var)))
+    
+    return (post_probs)
+    
+  }
+  
+  post_combs_i <- getPostCombsI(posterior_i)
+  post_probs   <- apply(contr$contMat, 2, getPostProb, post_combs_i)
+  
+  res <- c(sign          = ifelse(max(post_probs) > crit_prob_adj, 1, 0),
+           crit_prob_adj = crit_prob_adj,
+           max_post_prob = max(post_probs),
+           post_probs    = post_probs)
+  
+  return (res)
+  
 }
 
 #' @title getContr
 #'
 #' @description This function calculates contrast vectors that are optimal for detecting certain alternatives via applying the function optContr() of the DoseFinding package.
-#' Hereby 4 different options can be distinguished that are automatically executed based on the input that is provided
+#' Hereby, 4 different options can be distinguished that are automatically executed based on the input that is provided
 #' 1) Bayesian approach: If dose_weights and a prior_list are provided an optimized contrasts for the posterior sample size is calculated.
 #'    In detail,  in a first step the dose_weights (typically the number of patients per dose group) and the prior information is combined by calculating for
 #'    each dose group a posterior effective sample. Based on this posterior effective sample sizes the allocation ratio is derived, which allows for a calculation on
@@ -190,7 +297,6 @@ assessDesign <- function (
 #' dose_levels  <- c(0, 0.5, 2, 4, 8)
 #' mods <- DoseFinding::Mods(
 #'   linear      = NULL,
-#'   linlog      = NULL,
 #'   emax        = c(0.5, 1.2),
 #'   exponential = 2,
 #'   doses       = dose_levels,
@@ -206,76 +312,76 @@ assessDesign <- function (
 #'
 #' @export
 getContr <- function (
-
+    
   mods,
   dose_levels,
   dose_weights = NULL,
   prior_list   = NULL,
   sd_posterior = NULL,
   se_new_trial = NULL
-
+  
 ) {
-
+  
   checkmate::check_class(mods, classes = "Mods")
   checkmate::check_double(dose_levels, lower = 0, any.missing = FALSE, len = length(attr(mods, "doses")))
   checkmate::check_double(dose_weights, any.missing = FALSE, len = length(attr(mods, "doses")))
   checkmate::check_list(prior_list, names = "named", len = length(attr(mods, "doses")), any.missing = FALSE)
-
+  
   # frequentist & re-estimation
   if (!is.null(se_new_trial) &
       is.null(dose_weights) & is.null(prior_list) & is.null(sd_posterior)) {
-
+    
     w <- NULL
     S <- diag((se_new_trial)^2)
-
+    
     # frequentist & no re-estimation
   } else if (!is.null(dose_weights) &
              is.null(se_new_trial) & is.null(prior_list) & is.null(sd_posterior)) {
-
+    
     w <- dose_weights
     S <- NULL
-
+    
     # Bayesian & re-estimation
   } else if (!is.null(sd_posterior) &
              is.null(se_new_trial) & is.null(prior_list) & is.null(dose_weights)) {
-
+    
     w <- NULL
     S <- diag((sd_posterior)^2)
-
+    
     # Bayesian & no re-estimation
   } else if (!is.null(dose_weights) & !is.null(prior_list) &
              is.null(se_new_trial) & is.null(sd_posterior)) {
-
+    
     w <- dose_weights +
       suppressMessages(round(unlist(lapply(prior_list, RBesT::ess))))
     S <- NULL
-
+    
   } else {
-
+    
     stop (paste("Provided combiations of 'se_new_trial',",
                 "'dose_weights', 'prior_list', 'sd_posterior' not allowed.",
                 "See ?getContr for allowed combinations."))
-
+    
   }
-
+  
   if (is.null(w)) {
-
+    
     contr <- DoseFinding::optContr(
       models = mods,
       doses  = dose_levels,
       S      = S)
-
+    
   } else {
-
+    
     contr <- DoseFinding::optContr(
       models = mods,
       doses  = dose_levels,
       w      = w)
-
+    
   }
-
+  
   return (contr)
-
+  
 }
 
 #' @title getCritProb
@@ -296,7 +402,6 @@ getContr <- function (
 #'
 #' @examples
 #' mods <- DoseFinding::Mods(linear      = NULL,
-#'                           linlog      = NULL,
 #'                           emax        = c(0.5, 1.2),
 #'                           exponential = 2,
 #'                           doses       = c(0, 0.5, 2,4, 8))
@@ -310,33 +415,49 @@ getContr <- function (
 #'
 #' @export
 getCritProb <- function (
-
+    
   mods,
   dose_levels,
   dose_weights   = NULL,
   se_new_trial   = NULL,
   alpha_crit_val = 0.025
-
+  
 ) {
-
+  
   checkmate::check_class(mods, classes = "Mods")
   checkmate::check_double(dose_levels, lower = 0, any.missing = FALSE, len = length(dose_weights))
   checkmate::check_double(dose_weights, any.missing = FALSE, len = length(dose_levels))
   checkmate::check_double(alpha_crit_val, lower = 0, upper = 1)
-
+  
   contr <- getContr(mods           = mods,
                     dose_levels    = dose_levels ,
                     dose_weights   = dose_weights,
                     se_new_trial   = se_new_trial)
-
+  
   crit_prob <- stats::pnorm(DoseFinding::critVal(
     corMat      = contr$corMat,
     alpha       = alpha_crit_val,
     df          = 0,
     alternative = "one.sided"))
-
+  
   return (crit_prob)
+  
+}
 
+getModelSuccesses <- function (b_mcp) {
+  
+  stopifnot(inherits(b_mcp, "BayesianMCP"))
+  
+  model_indices <- grepl("post_probs.", colnames(b_mcp))
+  model_names   <- colnames(b_mcp)[model_indices] |>
+    sub(pattern = "post_probs.", replacement = "", x = _)
+  
+  model_successes <- colMeans(b_mcp[, model_indices] > b_mcp[, "crit_prob_adj"])
+  
+  names(model_successes) <- model_names
+  
+  return (model_successes)
+  
 }
 
 #' @title performBayesianMCPMod
@@ -347,9 +468,13 @@ getCritProb <- function (
 #' @param contr An object of class 'optContr' as created by the getContr() function. It contains the contrast matrix to be used for the testing step.
 #' @param crit_prob_adj A getCritProb object, specifying the critical value to be used for the testing (on the probability scale).
 #' @param simple Boolean variable, defining whether simplified fit will be applied. Passed to the getModelFits() function. Default FALSE.
+#' @param avg_fit Boolean variable, defining whether an average fit (based on generalized AIC weights) should be performed in addition to the individual models. Default TRUE.
+#' @param delta A numeric value for the threshold Delta for the MED assessment. If NULL, no MED assessment is performed. Default NULL.
+#' @param evidence_level A numeric value between 0 and 1 for the evidence level gamma for the MED assessment. Only required for Bayesian MED assessment, see ?getMED for details. Default NULL.
+#' @param med_selection A string, either "avgFit" or "bestFit" based on the lowest gAIC, for the method of MED selection. Default "avgFit".
+#' @param n_samples A numerical for the number of bootstrapped samples in case the Bayesian MED assessment is performed. Default 1e3.
 #' @examples
 #' mods <- DoseFinding::Mods(linear      = NULL,
-#'                           linlog      = NULL,
 #'                           emax        = c(0.5, 1.2),
 #'                           exponential = 2,
 #'                           doses       = c(0, 0.5, 2,4, 8))
@@ -363,7 +488,7 @@ getCritProb <- function (
 #'   mods           = mods,
 #'   dose_weights   = c(50, 50, 50, 50, 50), #reflecting the planned sample size
 #'   dose_levels    = dose_levels,
-#'   alpha_crit_val = 0.05)
+#'   alpha_crit_val = 0.6) # unreasonable alpha for this example, rather choose 0.05
 #' prior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 5), sigma = 2),
 #'                    DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 1, s = 12), sigma = 2),
 #'                    DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.2, s = 11), sigma = 2) ,
@@ -374,104 +499,145 @@ getCritProb <- function (
 #' posterior_list <- getPosterior(
 #'   prior_list = prior_list,
 #'   mu_hat     = mu,
-#'   S_hat     = S_hat)
+#'   S_hat      = S_hat,
+#'   calc_ess   = TRUE)
+#'                       
 #' performBayesianMCPMod(posterior_list = posterior_list,
 #'                       contr          = contr_mat,
 #'                       crit_prob_adj  = critVal,
-#'                       simple         = FALSE)
+#'                       simple         = FALSE,
+#'                       delta          = 1)
 #'
 #' @return Bayesian MCP test result as well as modeling result.
 #'
 #' @export
 performBayesianMCPMod <- function (
-
+    
   posterior_list,
   contr,
   crit_prob_adj,
-  simple = FALSE
-
+  simple = FALSE,
+  avg_fit = TRUE,
+  
+  delta          = NULL,
+  evidence_level = NULL,
+  med_selection  = c("avgFit", "bestFit"),
+  n_samples      = 1e3
+  
 ) {
-
+  
   checkmate::check_class(posterior_list, "postList")
   checkmate::check_class(contr, "optContr")
   checkmate::check_class(crit_prob_adj, "numeric")
   checkmate::check_logical(simple)
-
+  checkmate::check_logical(avg_fit)
+  
+  checkmate::check_double(delta)
+  checkmate::check_double(evidence_level, lower = 0, upper = 1)
+  checkmate::check_character(med_selection)
+  checkmate::check_double(n_samples, lower = 1)
+  
+  med_selection <- match.arg(med_selection, choices = c("avgFit", "bestFit"))
+  get_med       <- ifelse(is.null(delta), FALSE, TRUE)
+  
+  if (!is.null(evidence_level)) {
+    
+    stopifnot("delta must not be NULL if evidence_level is not NULL" = !is.null(delta))
+    
+  }
+  
   if (inherits(posterior_list,  "postList")) {
-
+    
     posterior_list <- list(posterior_list)
-
+    
   }
-
+  
   if (inherits(contr, "optContr")) {
-
-    model_shapes <- colnames(contr$contMat)
-    dose_levels  <- as.numeric(rownames(contr$contMat))
-
+    
+    model_names <- colnames(contr$contMat)
+    dose_levels <- as.numeric(rownames(contr$contMat))
+    
   } else if (length(contr) == length(posterior_list)) {
-
-    model_shapes <- colnames(contr[[1]]$contMat)
-    dose_levels  <- as.numeric(rownames(contr[[1]]$contMat))
-
+    
+    model_names <- colnames(contr[[1]]$contMat)
+    dose_levels <- as.numeric(rownames(contr[[1]]$contMat))
+    
   } else {
-
+    
     stop ("Argument 'contr' must be of type 'optContr'")
-
+    
   }
-
+  
   b_mcp <- performBayesianMCP(
     posterior_list = posterior_list,
     contr          = contr,
     crit_prob_adj  = crit_prob_adj)
-
-  fits_list <- lapply(seq_along(posterior_list), function (i) {
-
-    if (b_mcp[i, 1]) {
-
-      sign_models <- b_mcp[i, -c(1, 2)] > attr(b_mcp, "crit_prob_adj")
-
-      model_fits  <- getModelFits(
-        models      = model_shapes,
-        dose_levels = dose_levels,
-        posterior   = posterior_list[[i]],
-        simple      = simple)
-
-      model_fits  <- addSignificance(model_fits, sign_models)
-
-    } else {
-
-      NULL
-
-    }
-
-  })
-
-  bmcpmod        <- list(BayesianMCP = b_mcp, Mod = fits_list)
-  class(bmcpmod) <- "BayesianMCPMod"
-
-  return (bmcpmod)
-
-}
-
-addSignificance <- function (
-
-  model_fits,
-  sign_models
-
-) {
-
-  names(sign_models) <- NULL
-
-  model_fits_out <- lapply(seq_along(model_fits), function (i) {
-
-    c(model_fits[[i]], significant = sign_models[i])
-
-  })
-
-  attributes(model_fits_out) <- attributes(model_fits)
-
-  return (model_fits_out)
-
+  
+  b_mod <- performBayesianMod(
+    b_mcp          = b_mcp,
+    posterior_list = posterior_list,
+    model_names    = model_names,
+    dose_levels    = dose_levels,
+    simple         = simple,
+    avg_fit        = avg_fit)
+  
+  b_mcp_mod <- list(BayesianMCP = b_mcp, Mod = b_mod)
+  
+  if (get_med) {
+    
+    med_info <- t(sapply(b_mcp_mod$Mod, function (model_fits) {
+      
+      if (!is.null(model_fits)) {
+        
+        if (!is.null(evidence_level)) {
+          
+          bs_quantiles <- getBootstrapQuantiles(
+            model_fits = model_fits,
+            quantiles  = 1 - evidence_level,
+            n_samples  = n_samples)
+          
+          med_info <- getMED(
+            delta          = delta,
+            evidence_level = evidence_level,
+            bs_quantiles   = bs_quantiles)
+          
+        } else {
+          
+          med_info <- getMED(
+            delta      = delta,
+            model_fits = model_fits)
+          
+        }
+        
+        if (med_selection == "avgFit") {
+          
+          return (med_info[, "avgFit"])
+          
+        } else if (med_selection == "bestFit") {
+          
+          model_min <- which.min(sapply(model_fits, function (x) x$gAIC))
+          
+          return (med_info[, model_min])
+          
+        }
+        
+      } else {
+        
+        return (c(med_reached  = 0, med = NA))
+        
+      }
+      
+    }))
+    
+    attr(b_mcp_mod, "MED")          <- med_info
+    attr(b_mcp_mod, "MEDSelection") <- med_selection
+    
+  }
+  
+  class(b_mcp_mod) <- "BayesianMCPMod"
+  
+  return (b_mcp_mod)
+  
 }
 
 #' @title performBayesianMCP
@@ -486,7 +652,6 @@ addSignificance <- function (
 #'
 #' @examples
 #' mods <- DoseFinding::Mods(linear      = NULL,
-#'                           linlog      = NULL,
 #'                           emax        = c(0.5, 1.2),
 #'                           exponential = 2,
 #'                           doses       = c(0, 0.5, 2,4, 8))
@@ -511,7 +676,8 @@ addSignificance <- function (
 #' posterior_list <- getPosterior(
 #'   prior_list = prior_list,
 #'   mu_hat     = mu,
-#'   S_hat     = S_hat)
+#'   S_hat      = S_hat,
+#'   calc_ess   = TRUE)
 #'
 #' performBayesianMCP(posterior_list = posterior_list,
 #'                    contr          = contr_mat,
@@ -521,34 +687,34 @@ addSignificance <- function (
 #'
 #' @export
 performBayesianMCP <- function(
-
+    
   posterior_list,
   contr,
   crit_prob_adj
-
+  
 ) {
-
+  
   checkmate::check_class(posterior_list, "postList")
   checkmate::check_class(contr, "optContr")
   checkmate::check_class(crit_prob_adj, "numeric")
   checkmate::check_numeric(crit_prob_adj, lower = 0, upper = Inf)
-
+  
   if (inherits(posterior_list,  "postList")) {
-
+    
     posterior_list <- list(posterior_list)
-
+    
   }
-
+  
   if (inherits(contr, "optContr")) {
-
+    
     b_mcp <- t(sapply(posterior_list, BayesMCPi, contr, crit_prob_adj))
-
+    
   } else {
-
+    
     b_mcp <- t(mapply(BayesMCPi, posterior_list, contr, crit_prob_adj))
-
+    
   }
-
+  
   class(b_mcp)               <- "BayesianMCP"
   attr(b_mcp, "critProbAdj") <- crit_prob_adj
   attr(b_mcp, "successRate") <- mean(b_mcp[, 1])
@@ -556,70 +722,46 @@ performBayesianMCP <- function(
     test = is.na(attr(posterior_list[[1]], "ess")),
     yes  = numeric(0),
     no   = rowMeans(sapply(posterior_list, function (posteriors) {
-
+      
       attr(posteriors, "ess")
-
+      
     })))
-
+  
   return (b_mcp)
-
+  
 }
 
-getModelSuccesses <- function (b_mcp) {
-
-  stopifnot(inherits(b_mcp, "BayesianMCP"))
-
-  model_indices <- grepl("post_probs.", colnames(b_mcp))
-  model_names   <- colnames(b_mcp)[model_indices] |>
-    sub(pattern = "post_probs.", replacement = "", x = _)
-
-  model_successes <- colMeans(b_mcp[, model_indices] > b_mcp[, "crit_prob_adj"])
-
-  names(model_successes) <- model_names
-
-  return (model_successes)
-
-}
-
-BayesMCPi <- function (
-
-  posterior_i,
-  contr,
-  crit_prob_adj
-
+performBayesianMod <- function (
+    
+  b_mcp,
+  posterior_list,
+  model_names,
+  dose_levels,
+  simple,
+  avg_fit
+  
 ) {
-
-  getPostProb <- function (
-
-    contr_j,     # j: dose level
-    post_combs_i # i: simulation outcome
-
-  ) {
-
-    ## Test statistic = sum over all components of
-    ## posterior weight * normal probability distribution of
-    ## critical values for doses * estimated mean / sqrt(product of critical values for doses)
-
-    ## Calculation for each component of the posterior
-    contr_theta   <- apply(post_combs_i$means, 1, `%*%`, contr_j)
-    contr_var     <- apply(post_combs_i$vars, 1, `%*%`, contr_j^2)
-    contr_weights <- post_combs_i$weights
-
-    ## P(c_m * theta > 0 | Y = y) for a shape m (and dose j)
-    post_probs <- sum(contr_weights * stats::pnorm(contr_theta / sqrt(contr_var)))
-
-    return (post_probs)
-
-  }
-
-  post_combs_i <- getPostCombsI(posterior_i)
-  post_probs   <- apply(contr$contMat, 2, getPostProb, post_combs_i)
-
-  res <- c(sign          = ifelse(max(post_probs) > crit_prob_adj, 1, 0),
-           crit_prob_adj = crit_prob_adj,
-           max_post_prob = max(post_probs),
-           post_probs    = post_probs)
-
-  return (res)
-
+  
+  fits_list <- future.apply::future_lapply(seq_along(posterior_list), function (i) {
+    
+    if (b_mcp[i, 1]) {
+      
+      model_fits <- getModelFits(
+        models      = model_names,
+        dose_levels = dose_levels,
+        posterior   = posterior_list[[i]],
+        avg_fit     = avg_fit,
+        simple      = simple) 
+      
+      sign_models <- b_mcp[i, -c(1, 2)] > attr(b_mcp, "critProbAdj")
+      model_fits  <- addSignificance(model_fits, sign_models)
+      
+    } else {
+      
+      NULL
+      
+    }
+    
+  })
+  
 }
